{% extends "base.html" %}

{% block title %}Imprimatur &gt; Run {{run_id}}{% endblock %}

{% block nav %}
  <a href="/">Imprimatur</a> &gt; Run {{run_id}}
{% endblock %}

{% block content %}
  <h2>Results</h2>
  <pre>{{proc.results_str()}}</pre>

  {% if proc.is_alive() %}
    <p>
      The script is still running. Refresh the page to see latest progress...
    </p>
  {% endif %}

  <h2>Script</h2>
  <pre>{{proc.script}}</pre>
{% endblock %}
