<!DOCTYPE html>
<html>
  <head>
    <title>{% block title %}{% endblock %}</title>
  </head>
  <body>
    <p>
      {% block nav %}{% endblock %}
    </p>

    {% block content %}{% endblock %}
  </body>
</html>

